mod colormap;

use std::env;
use std::fs::File;
use std::io::{self, Read};

use colormap::{map_char_to_color, map_u8_to_color};

const GLOBAL_BUFFER_LENGTH: usize = 16;

fn dump_to_hex(bytes: &mut [u8]) -> String {
    let out_vec: Vec<String> = bytes
        .chunks(2)
        .map(|c| {
            if c.len() == 2 {
                format!("{}{}", map_u8_to_color(c[0]), map_u8_to_color(c[1]))
            } else {
                map_u8_to_color(c[0])
            }
        })
        .collect();
    out_vec.join(" ")
}

fn dump_to_chr(bytes: &mut [u8]) -> String {
    let out_vec: Vec<String> = bytes.iter().map(|ord| map_char_to_color(*ord)).collect();
    out_vec.join("")
}

fn main() {
    let args: Vec<String> = env::args().collect();

    let mut reader: Box<dyn io::Read> = if args.len() == 1 {
        Box::new(io::stdin())
    } else {
        let filename = &args[1];
        Box::new(File::open(filename).expect("Could not open file."))
    };

    let mut buf = [0; GLOBAL_BUFFER_LENGTH];
    let mut offset: usize = 0;

    loop {
        let bytes_read = reader.read(&mut buf);
        match bytes_read {
            Ok(num) => {
                if num == 0 {
                    break;
                } else {
                    println!(
                        "{:08x}: {:40} {:10}",
                        offset,
                        dump_to_hex(&mut buf[0..num]),
                        dump_to_chr(&mut buf[0..num])
                    );
                    offset += GLOBAL_BUFFER_LENGTH;
                }
            }
            Err(e) => {
                eprintln!("hd: {}", e);
                break;
            }
        }
    }
}
